#!/usr/bin/php
<?php
define(NEWLINE, "\n");

if(count($argv) <= 1){
	die(
'
*****************************************************
Language Builder Script 
for the SilverStripe Module Author.
Author:  Roman Schmid, aka Banal
Version: 1.0
*****************************************************
This script searches all .php and .ss files in a given 
directory for calls to the translate function _t.
All found instances will be saved as a php language file

Usage:  LangBuilder.php <dir> [<outfile>]
dir     The directory to search for files. Usually this
        should be your module directory.
outfile Optional. where to save the found translation
        entities to. Defaults to <dir>/lang/en_US.php
        
Examples:
LangBuilder.php mymodule
Will search the "mymodule" folder that\'s in the same 
directory as the LangBuilder.php file. Will extract
all translatable entities and store them in 
mymodule/lang/en_US.php
*****************************************************
'
	);
}

$dir = $argv[1];
if(!is_dir($dir)){
	die('ERROR: Not a valid directory: '. $dir . NEWLINE);
}

if(trim($dir) === '/')
	die('ERROR: You must be joking' . NEWLINE);

echo 'Performing search on: ' . $dir . NEWLINE;
$realdir = realpath($dir);
$dest = isset($argv[2]) ? $argv[2] : $realdir . '/lang/en_US.php';

$base = dirname($base);
if(is_dir($base))
	die('ERROR: Output directory: ' . $base . ' does not exist!' . NEWLINE);


$extensions = array('.php', '.ss');
$entries = array();

function searchContent($file)
{
	global $entries;
	echo 'Searching content of file: ' . $file . NEWLINE;

	$handle = fopen($file, 'r');
	$content = fread($handle, filesize($file));
	fclose($handle);
	
	if(!$content){
		echo '-- Could not read contents of file!' . NEWLINE;
		return;
	}
	
	$matches = array();
	// match all translatable entities. Entry 1,3,4 and 6 are to ignore (quotes)
	// the entries we need are in 2 (entity name) and 5 (translatable string)
	$found = preg_match_all('{_t\s*\(\s*(\'|")(.*?)(\1)\s*,\s*(\'|")(.*?)(\4)}msi', $content, $matches);
	if(!$found){
		echo '-- No translatable entities found in file.' . NEWLINE;
		return;
	}
	
	for($i = 0; $i < $found; $i++){
		$key = $matches[2][$i];
		
		// find the entity name.
		$entity = strrchr($key, '.');
		
		// if there's no class in the entity name, use the filename
		if(trim($entity) === ''){
			$entity = $key;
			$key = basename($file);
		} else {
			$key = substr($key, 0, -strlen($entity));
			$entity = ltrim($entity, '.');
		}
		
		if(!isset($entries[$key]))
			$entries[$key] = array();
			
		if(!isset($entries[$key][$entity]))
			$entries[$key][$entity] = $matches[5][$i];
	}
	//print_r($matches);
	
}

function replaceDir($searchDir)
{
	global $extensions;
	echo 'Entering directory: '. $searchDir . NEWLINE;
	$handle = opendir($searchDir);
	while (($file = readdir($handle)) !== false) {
		// skip dot and double-dot files
		if($file === '.' || $file === '..')
			continue;
		
		$fpath = $searchDir . '/' . $file;
		
		if(is_file($fpath)){
			$ext = strtolower(strrchr($file, '.'));
			if(in_array($ext, $extensions)){
				searchContent($fpath);
			}
		} else if(is_dir($fpath)){
			replaceDir($fpath);
		}
	}
}

// recursively search all sub-directories
replaceDir($realdir);

// now we have a populated $entries array
$output = '<?php
// -------------------------------------------------------------------
// This file was automatically generated by the LangBuilder.php script
// Date of creation: '.date('Y/m/d H:i:s').'
// -------------------------------------------------------------------
global $lang;

';
foreach($entries as $key => $arr){
	$output .= NEWLINE . '// Output for class or file: '. $key . NEWLINE;
	foreach($arr as $entry => $trnl){
		$output .= '$lang[\'en_US\'][\''. $key .'\'][\''. $entry .'\'] = ' . NEWLINE;
		$output .= '	\''. str_replace("'", "\'", $trnl) . '\';' . NEWLINE; 
	}
}

$handle = fopen($dest, 'w');
echo '---------------------------------------------------' . NEWLINE;
if(fwrite($handle, $output) === false){
	echo 'ERROR: Unable to write to: ' . $dest . NEWLINE;
} else {
	echo 'Language entities successfully written to: ' . NEWLINE . $dest . NEWLINE;
}
echo '---------------------------------------------------' . NEWLINE;
fclose($handle);

?>